import 'dart:convert';

import 'package:meta/meta.dart';

enum Quoting {
  singleQuote(r"'"),
  doubleQuote(r'"');

  final String quote;

  const Quoting(this.quote);
}

@internal
extension StringNotation on String {
  (String, String) get wrappedChar => (this[0], this[length - 1]);

  bool get canBeParsed {
    if ([num.tryParse(this), bool.tryParse(this)]
        .any((element) => element != null)) {
      return true;
    }

    final (firstChar, lastChar) = wrappedChar;
    final bool isArray = firstChar == r"[" && lastChar == r"]";
    final bool isObject = firstChar == r"{" && lastChar == r"}";

    return isArray || isObject;
  }

  bool isQuotedWith(Quoting quoting) {
    final (firstChar, lastChar) = wrappedChar;

    return firstChar == quoting.quote && lastChar == quoting.quote;
  }

  bool get useQuote => Quoting.values.any(isQuotedWith);
}

final class EscapedCharCodec extends Codec<String, String> {
  final Quoting? quoting;

  const EscapedCharCodec({this.quoting});

  @override
  Converter<String, String> get decoder => const _EscapedCharDecoder();

  @override
  Converter<String, String> get encoder => quoting == null
      ? const _EscapedCharEncoder()
      : _EscapedCharEncoder(quoting!);
}

final class _EscapedCharDecoder extends Converter<String, String> {
  const _EscapedCharDecoder();

  @override
  String convert(String input) {
    String decCtx = input;

    if (decCtx.useQuote) {
      decCtx = decCtx.substring(1, decCtx.length - 1);
    }

    decCtx = jsonDecode('"$decCtx"');

    if (input.useQuote) {
      decCtx = '"$decCtx"';
    }

    return decCtx;
  }
}

final class _EscapedCharEncoder extends Converter<String, String> {
  final Quoting quoting;

  const _EscapedCharEncoder([this.quoting = Quoting.doubleQuote]);

  @override
  String convert(String input) {
    String encoded = jsonEncode(input);
    encoded = encoded.substring(1, encoded.length - 1);

    if (encoded.canBeParsed) {
      encoded = quoting.quote + encoded + quoting.quote;
    }

    return encoded;
  }
}
